==> grabberlib2-0.1.22.tar.gz: grabberlib2-0.1.22/src/grabber/core/sources/lovecos.py <==
import pathlib
import uuid
from typing import List, Optional

from ada_url import replace_url
from boltons.setutils import IndexedSet
from telegraph import Telegraph
from tqdm import tqdm
from url_parser import get_url, get_base_url

from database.models import ExtractedPage
from database.repositories.extracted_page import ExtractedPageRepository
from grabber.core.utils import (
    build_unique_img_urls,
    get_all_posts_for_url,
    run_downloader,
    get_soup,
    get_tags,
    headers_mapping,
    query_mapping,
    send_post_to_telegram,
)


async def get_pages_from_pagination(url: str, headers: Optional[dict] = None) -> List[str]:
    pagination_query = "div.page a"
    images_pages_query = "div.mimg div.mtp li a"
    parsed_url = get_url(url)
    base_url = get_base_url(url)

    source_urls = IndexedSet()
    source_urls.add(url)
    soup = await get_soup(url, headers=headers)
    pagination_set = soup.select(pagination_query)
    images_pages_tags = soup.select(images_pages_query)
    images_links = IndexedSet()

    for image_tag in images_pages_tags:
        href = image_tag.attrs["href"]
        images_links.add(replace_url(url, pathname=href))

    if not pagination_set:
        return list(images_links)

    pages = []

    for a in pagination_set:
        text: str = a.attrs["href"]
        pages.append(text)

    for href in pages:
        target_url = f"{base_url}{parsed_url.dir}{href}"
        source_urls.add(target_url)

    for url in list(source_urls)[1:]:
        soup = await get_soup(url, headers=headers)
        images_pages_tags = soup.select(images_pages_query)
        for image_tag in images_pages_tags:
            href = image_tag.attrs["href"]
            images_links.add(replace_url(url, pathname=href))

    return list(images_links)


async def get_sources_for_lovecos(
    sources: List[str],
    entity: str,
    telegraph_client: Optional[Telegraph] = None,
    final_dest: str | pathlib.Path = "",
    save_to_telegraph: bool | None = False,
    is_tag: Optional[bool] = False,
    limit: Optional[int] = None,
    **kwargs,
) -> None:
    tqdm_sources_iterable = tqdm(
        sources,
        total=len(sources),
        desc="Retrieving URLs...",
    )
    query, src_attr = query_mapping[entity]
    headers = headers_mapping.get(entity, None)
    page_title = ""
    title_folder_mapping = {}
    posts_sent_counter = 0
    titles = IndexedSet()
    repository = ExtractedPageRepository(model=ExtractedPage)
    ordered_unique_img_urls = None
    all_sources = []
    original_folder_path = final_dest

    for source_url in tqdm_sources_iterable:
        final_dest = pathlib.Path(str(original_folder_path))
        all_sources += await get_all_posts_for_url(repository=repository, url=source_url)

        if source_url in all_sources:
            tqdm_sources_iterable.set_description(
                f"Skipping {source_url} since it was already posted"
            )
            continue

        folder_name = ""
        urls = [
            *await get_pages_from_pagination(
                url=source_url,
                headers=headers,
            ),
        ]
        image_tags = []

        for index, url in enumerate(urls):
            tags, soup = await get_tags(
                url,
                headers=headers,
                query=query,
            )
            image_tags.extend(tags or [])
            page_title = (
                soup.find("title")
                .get_text(strip=True)
                .split(",www.lovecos.net")[0]
                .split(",  www.lovecos.net")[0]
                .split(",")[0]
                .strip()
                .rstrip()
            )

            if index == 0 or not page_title:
                page_title = (
                    soup.find("title")
                    .get_text(strip=True)
                    .split(",www.lovecos.net")[0]
                    .split(",  www.lovecos.net")[0]
                    .split(",")[0]
                    .strip()
                    .rstrip()
                )
                folder_name = f"{page_title}"
                titles.add(page_title)

        ordered_unique_img_urls = await build_unique_img_urls(image_tags, src_attr)
        tqdm_sources_iterable.set_description(f"Finished retrieving images for {page_title}")

        if final_dest:
            final_dest = pathlib.Path(final_dest) / f"{page_title} - {entity}"
            if not final_dest.exists():
                final_dest.mkdir(parents=True, exist_ok=True)

            folder_name = f"{page_title}"
            # title_dest = final_dest / folder_name / f"{str(uuid.uuid4())}"
            # if not title_dest.exists():
                # title_dest.mkdir(parents=True, exist_ok=True)
            title_folder_mapping[page_title] = (ordered_unique_img_urls, final_dest)

        if save_to_telegraph:
            all_sources = await send_post_to_telegram(
                ordered_unique_img_urls=ordered_unique_img_urls,
                page_title=page_title,
                telegraph_client=telegraph_client,
                posts_sent_counter=posts_sent_counter,
                tqdm_sources_iterable=tqdm_sources_iterable,
                all_sources=all_sources,
                source_url=source_url,
                entity=entity,
            )
        page_title = ""

    if final_dest and ordered_unique_img_urls:
        await run_downloader(
            final_dest=final_dest,
            page_title=page_title,
            unique_img_urls=ordered_unique_img_urls,
            titles=titles,
            title_folder_mapping=title_folder_mapping,
            headers=headers,
        )

==> grabberlib2-0.1.24.tar.gz: grabberlib2-0.1.24/src/grabber/core/sources/lovecos.py <==
import pathlib
import uuid
from typing import List, Optional

from ada_url import replace_url
from boltons.setutils import IndexedSet
from telegraph import Telegraph
from tqdm import tqdm
from url_parser import get_url, get_base_url
from unidecode import unidecode

from database.models import ExtractedPage
from database.repositories.extracted_page import ExtractedPageRepository
from grabber.core.utils import (
    build_unique_img_urls,
    get_all_posts_for_url,
    run_downloader,
    get_soup,
    get_tags,
    headers_mapping,
    query_mapping,
    send_post_to_telegram,
)


async def get_pages_from_pagination(url: str, headers: Optional[dict] = None) -> List[str]:
    pagination_query = "div.page a"
    images_pages_query = "div.mimg div.mtp li a"
    parsed_url = get_url(url)
    base_url = get_base_url(url)

    source_urls = IndexedSet()
    source_urls.add(url)
    soup = await get_soup(url, headers=headers)
    pagination_set = soup.select(pagination_query)
    images_pages_tags = soup.select(images_pages_query)
    images_links = IndexedSet()

    for image_tag in images_pages_tags:
        href = image_tag.attrs["href"]
        images_links.add(replace_url(url, pathname=href))

    if not pagination_set:
        return list(images_links)

    pages = []

    for a in pagination_set:
        text: str = a.attrs["href"]
        pages.append(text)

    for href in pages:
        target_url = f"{base_url}{parsed_url.dir}{href}"
        source_urls.add(target_url)

    for url in list(source_urls)[1:]:
        soup = await get_soup(url, headers=headers)
        images_pages_tags = soup.select(images_pages_query)
        for image_tag in images_pages_tags:
            href = image_tag.attrs["href"]
            images_links.add(replace_url(url, pathname=href))

    return list(images_links)


async def get_sources_for_lovecos(
    sources: List[str],
    entity: str,
    telegraph_client: Optional[Telegraph] = None,
    final_dest: str | pathlib.Path = "",
    save_to_telegraph: bool | None = False,
    is_tag: Optional[bool] = False,
    limit: Optional[int] = None,
    **kwargs,
) -> None:
    tqdm_sources_iterable = tqdm(
        sources,
        total=len(sources),
        desc="Retrieving URLs...",
    )
    query, src_attr = query_mapping[entity]
    headers = headers_mapping.get(entity, None)
    page_title = ""
    title_folder_mapping = {}
    posts_sent_counter = 0
    titles = IndexedSet()
    repository = ExtractedPageRepository(model=ExtractedPage)
    ordered_unique_img_urls = None
    all_sources = []
    original_folder_path = final_dest

    for source_url in tqdm_sources_iterable:
        final_dest = pathlib.Path(str(original_folder_path))
        all_sources += await get_all_posts_for_url(repository=repository, url=source_url)

        if source_url in all_sources:
            tqdm_sources_iterable.set_description(
                f"Skipping {source_url} since it was already posted"
            )
            continue

        folder_name = ""
        urls = [
            *await get_pages_from_pagination(
                url=source_url,
                headers=headers,
            ),
        ]
        image_tags = []

        for index, url in enumerate(urls):
            tags, soup = await get_tags(
                url,
                headers=headers,
                query=query,
            )
            image_tags.extend(tags or [])
            page_title = (
                soup.find("title")
                .get_text(strip=True)
                .split(",www.lovecos.net")[0]
                .split(",  www.lovecos.net")[0]
                .split(",")[0]
                .strip()
                .rstrip()
            )

            if index == 0 or not page_title:
                page_title = (
                    soup.find("title")
                    .get_text(strip=True)
                    .split(",www.lovecos.net")[0]
                    .split(",  www.lovecos.net")[0]
                    .split(",")[0]
                    .strip()
                    .rstrip()
                )
                folder_name = f"{page_title}"
                titles.add(page_title)

        ordered_unique_img_urls = await build_unique_img_urls(image_tags, src_attr)
        tqdm_sources_iterable.set_description(f"Finished retrieving images for {page_title}")

        if final_dest:
            final_dest = pathlib.Path(final_dest) / unidecode(f"{page_title} - {entity}")
            if not final_dest.exists():
                final_dest.mkdir(parents=True, exist_ok=True)

            folder_name = f"{page_title}"
            # title_dest = final_dest / folder_name / f"{str(uuid.uuid4())}"
            # if not title_dest.exists():
                # title_dest.mkdir(parents=True, exist_ok=True)
            title_folder_mapping[page_title] = (ordered_unique_img_urls, final_dest)

        if save_to_telegraph:
            all_sources = await send_post_to_telegram(
                ordered_unique_img_urls=ordered_unique_img_urls,
                page_title=page_title,
                telegraph_client=telegraph_client,
                posts_sent_counter=posts_sent_counter,
                tqdm_sources_iterable=tqdm_sources_iterable,
                all_sources=all_sources,
                source_url=source_url,
                entity=entity,
            )
        page_title = ""

    if final_dest and ordered_unique_img_urls:
        await run_downloader(
            final_dest=final_dest,
            page_title=page_title,
            unique_img_urls=ordered_unique_img_urls,
            titles=titles,
            title_folder_mapping=title_folder_mapping,
            headers=headers,
        )
